import fs from 'fs';
import path from 'path';
import { isReact18, normalizeToPosixPath } from '@modern-js/utils';
import { FILE_SYSTEM_ROUTES_FILE_NAME } from "./constants";
export const walkDirectory = dir => fs.readdirSync(dir).reduce((previous, filename) => {
  const filePath = path.join(dir, filename);

  if (fs.statSync(filePath).isDirectory()) {
    return [...previous, ...walkDirectory(filePath)];
  } else {
    return [...previous, filePath];
  }
}, []);
export const getDefaultImports = ({
  entrypoint,
  srcDirectory,
  internalSrcAlias,
  internalDirAlias,
  internalDirectory
}) => {
  const {
    entryName,
    fileSystemRoutes,
    customBootstrap,
    entry
  } = entrypoint;
  const imports = [{
    specifiers: [{
      local: 'React'
    }],
    value: 'react'
  }, {
    specifiers: [{
      local: 'ReactDOM'
    }],
    value: isReact18(path.join(internalDirectory, '../../')) ? 'react-dom/client' : 'react-dom'
  }, {
    specifiers: [{
      imported: 'createApp'
    }, {
      imported: 'bootstrap'
    }],
    value: '@modern-js/runtime'
  }, customBootstrap && {
    specifiers: [{
      local: 'customBootstrap'
    }],
    value: normalizeToPosixPath(customBootstrap.replace(srcDirectory, internalSrcAlias))
  }].filter(Boolean);

  if (fileSystemRoutes) {
    const route = {
      specifiers: [{
        imported: 'routes'
      }],
      value: normalizeToPosixPath(`${internalDirAlias}/${entryName}/${FILE_SYSTEM_ROUTES_FILE_NAME}`)
    };

    if (fileSystemRoutes.globalApp) {
      imports.push({
        specifiers: [{
          local: 'App'
        }],
        value: normalizeToPosixPath(fileSystemRoutes.globalApp.replace(srcDirectory, internalSrcAlias))
      });
    } else {
      route.initialize = 'const App = false;';
    }

    imports.push(route);
  } else {
    imports.push({
      specifiers: [{
        local: 'App'
      }],
      value: normalizeToPosixPath(entry.replace(srcDirectory, internalSrcAlias))
    });
  }

  return imports;
};
export const isRouteComponentFile = filePath => {
  if (/\.(d|test|spec|e2e)\.(js|jsx|ts|tsx)$/.test(filePath)) {
    return false;
  }

  if (['.js', '.jsx', '.ts', '.tsx'].includes(path.extname(filePath))) {
    return true;
  }

  return false;
};