function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { fs, logger, chalk, isSSR } from '@modern-js/utils';
import { ResolvedConfigContext } from '@modern-js/core';
import { createCompiler } from "../utils/createCompiler";
import { createServer } from "../utils/createServer";
import { generateRoutes } from "../utils/routes";
import { printInstructions } from "../utils/printInstructions";
import { getSpecifiedEntries } from "../utils/getSpecifiedEntries";
import { buildServerConfig } from "../utils/config";
export const dev = async (api, options) => {
  let userConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const hookRunners = api.useHookRunners();
  userConfig = _objectSpread(_objectSpread({}, userConfig), {}, {
    cliOptions: options
  });
  ResolvedConfigContext.set(userConfig);
  const {
    appDirectory,
    distDirectory,
    port,
    apiOnly,
    entrypoints,
    serverConfigFile
  } = appContext;
  const checkedEntries = await getSpecifiedEntries(options.entry || false, entrypoints);
  api.setAppContext(_objectSpread(_objectSpread({}, appContext), {}, {
    checkedEntries
  }));
  appContext.checkedEntries = checkedEntries;
  fs.emptyDirSync(distDirectory);
  await buildServerConfig({
    appDirectory,
    distDirectory,
    configFile: serverConfigFile,
    options: {
      esbuildOptions: {
        watch: true
      }
    }
  });
  await hookRunners.beforeDev();
  let compiler = null;

  if (!apiOnly) {
    const {
      getWebpackConfig,
      WebpackConfigTarget
    } = await import('@modern-js/webpack');
    const webpackConfigs = [isSSR(userConfig) && getWebpackConfig(WebpackConfigTarget.NODE, appContext, userConfig), getWebpackConfig(WebpackConfigTarget.CLIENT, appContext, userConfig)].filter(Boolean);
    compiler = await createCompiler({
      api,
      webpackConfigs,
      userConfig,
      appContext
    });
  }

  await generateRoutes(appContext);
  const app = await createServer({
    dev: _objectSpread(_objectSpread({}, {
      client: {
        port: port.toString()
      },
      devMiddleware: {
        writeToDisk: file => !file.includes('.hot-update.')
      },
      hot: true,
      liveReload: true,
      port,
      https: userConfig.dev.https
    }), userConfig.tools.devServer),
    compiler,
    pwd: appDirectory,
    config: userConfig,
    serverConfigFile,
    plugins: appContext.plugins.filter(p => p.server).map(p => p.server)
  });
  app.listen(port, async err => {
    if (err) {
      throw err;
    }

    if (apiOnly) {
      return printInstructions(hookRunners, appContext, userConfig);
    }

    return logger.log(chalk.cyan(`Starting the development server...`));
  });
};