import path from 'path';
import { getWebpackConfig, WebpackConfigTarget } from '@modern-js/webpack';
import { fs, logger, isUseSSRBundle, chalk } from '@modern-js/utils';
import WebpackChain from '@modern-js/utils/webpack-chain';
export const formatWebpackConfig = (config, verbose) => {
  const stringify = WebpackChain.toString;
  return `module.exports = ${stringify(config, {
    verbose
  })};`;
};
export const inspect = (api, options) => {
  process.env.NODE_ENV = options.env;
  const resolvedConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const outputFiles = [];
  outputFiles.push(printInspectResult(WebpackConfigTarget.CLIENT, appContext, resolvedConfig, options));

  if (resolvedConfig.output.enableModernMode) {
    outputFiles.push(printInspectResult(WebpackConfigTarget.MODERN, appContext, resolvedConfig, options));
  }

  if (isUseSSRBundle(resolvedConfig)) {
    outputFiles.push(printInspectResult(WebpackConfigTarget.NODE, appContext, resolvedConfig, options));
  }

  logger.success('Inspect succeed, you can open following files to view the full webpack config: \n');
  outputFiles.forEach(file => {
    logger.log(`  - ${chalk.yellow(path.relative(appContext.appDirectory, file))}`);
  });
  logger.log();
};
export const getTagByWebpackTarget = webpackTarget => {
  switch (webpackTarget) {
    case WebpackConfigTarget.CLIENT:
      return 'client';

    case WebpackConfigTarget.MODERN:
      return 'modern';

    case WebpackConfigTarget.NODE:
      return 'ssr';

    default:
      throw Error(`Unsupported webpack target: ${webpackTarget}`);
  }
};
export const printInspectResult = (webpackTarget, appContext, resolvedConfig, options) => {
  const webpackConfig = getWebpackConfig(webpackTarget, appContext, resolvedConfig);
  const {
    output,
    verbose,
    console = true
  } = options;
  const outputPath = output ? path.posix.join(appContext.distDirectory, output) : appContext.distDirectory;
  const tag = getTagByWebpackTarget(webpackTarget);
  const outputFile = `webpack.${tag}.inspect.js`;
  const outputFilePath = path.posix.join(outputPath, outputFile);
  const rawWebpackConfig = formatWebpackConfig(webpackConfig, verbose);
  fs.outputFileSync(outputFilePath, rawWebpackConfig);

  if (console) {
    logger.log(`
webpack config for ${tag} build:
${rawWebpackConfig}
    `);
  }

  return outputFilePath;
};