function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import path from 'path';
import { defineConfig, cli } from '@modern-js/core';
import LintPlugin from '@modern-js/plugin-jarvis';
import { cleanRequireCache, Import } from '@modern-js/utils';
import AnalyzePlugin from "./analyze";
import { hooks } from "./hooks";
import { i18n, localeKeys } from "./locale";
import { getLocaleLanguage } from "./utils/language";
export { defineConfig };
const upgradeModel = Import.lazy('@modern-js/upgrade', require);
export default (() => ({
  name: '@modern-js/app-tools',
  post: ['@modern-js/plugin-analyze', '@modern-js/plugin-ssr', '@modern-js/plugin-state', '@modern-js/plugin-router', '@modern-js/plugin-polyfill'],
  registerHook: hooks,
  usePlugins: [AnalyzePlugin(), LintPlugin()],
  setup: api => {
    const locale = getLocaleLanguage();
    i18n.changeLanguage({
      locale
    });
    return {
      commands({
        program
      }) {
        program.command('dev').usage('[options]').description(i18n.t(localeKeys.command.dev.describe)).option('-c --config <config>', i18n.t(localeKeys.command.shared.config)).option('-e --entry [entry...]', i18n.t(localeKeys.command.dev.entry)).option('--analyze', i18n.t(localeKeys.command.shared.analyze)).option('--api-only', i18n.t(localeKeys.command.dev.apiOnly)).action(async options => {
          const {
            dev
          } = await import("./commands/dev");
          await dev(api, options);
        });
        program.command('build').usage('[options]').description(i18n.t(localeKeys.command.build.describe)).option('-c --config <config>', i18n.t(localeKeys.command.shared.config)).option('--analyze', i18n.t(localeKeys.command.shared.analyze)).action(async options => {
          const {
            build
          } = await import("./commands/build");
          await build(api, options);
        });
        program.command('start').usage('[options]').description(i18n.t(localeKeys.command.start.describe)).option('--api-only', i18n.t(localeKeys.command.dev.apiOnly)).option('-c --config <config>', i18n.t(localeKeys.command.shared.config)).action(async () => {
          const {
            start
          } = await import("./commands/start");
          await start(api);
        });
        program.command('deploy').usage('[options]').option('-c --config <config>', i18n.t(localeKeys.command.shared.config)).description(i18n.t(localeKeys.command.deploy.describe)).action(async options => {
          const {
            build
          } = await import("./commands/build");
          await build(api);
          const {
            deploy
          } = await import("./commands/deploy");
          await deploy(api, options); // eslint-disable-next-line no-process-exit

          process.exit(0);
        });
        program.command('new').usage('[options]').description(i18n.t(localeKeys.command.new.describe)).option('-d, --debug', i18n.t(localeKeys.command.new.debug), false).option('-c, --config <config>', i18n.t(localeKeys.command.new.config)).option('--dist-tag <tag>', i18n.t(localeKeys.command.new.distTag)).option('--registry', i18n.t(localeKeys.command.new.registry)).action(async options => {
          const {
            MWANewAction
          } = await import('@modern-js/new-action');
          await MWANewAction(_objectSpread(_objectSpread({}, options), {}, {
            locale
          }));
        });
        program.command('inspect').description('inspect internal webpack config').option(`--env <env>`, i18n.t(localeKeys.command.inspect.env), 'development').option('--output <output>', i18n.t(localeKeys.command.inspect.output), '/').option('--no-console', i18n.t(localeKeys.command.inspect.noConsole)).option('--verbose', i18n.t(localeKeys.command.inspect.verbose)).option('-c --config <config>', i18n.t(localeKeys.command.shared.config)).action(async options => {
          const {
            inspect
          } = await import("./commands/inspect");
          inspect(api, options);
        });
        upgradeModel.defineCommand(program.command('upgrade'));
      },

      // 这里会被 core/initWatcher 监听的文件变动触发，如果是 src 目录下的文件变动，则不做 restart
      async fileChange(e) {
        const {
          filename,
          eventType
        } = e;
        const appContext = api.useAppContext();
        const {
          appDirectory,
          srcDirectory
        } = appContext;
        const absolutePath = path.resolve(appDirectory, filename);

        if (!absolutePath.includes(srcDirectory) && (eventType === 'change' || eventType === 'unlink')) {
          const {
            closeServer
          } = await import("./utils/createServer");
          await closeServer();
          await cli.restart();
        }
      },

      async beforeRestart() {
        cleanRequireCache([require.resolve("./analyze")]);
      }

    };
  }
}));