function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import * as path from 'path';
import { bundle } from '@modern-js/node-bundle-require';
import { CONFIG_FILE_EXTENSIONS, fs, getServerConfig, OUTPUT_CONFIG_FILE } from '@modern-js/utils';
export const defineServerConfig = config => config;
export const buildServerConfig = async ({
  appDirectory,
  distDirectory,
  configFile,
  options
}) => {
  const configFilePath = await getServerConfig(appDirectory, configFile);

  const getOutputFile = async filepath => path.resolve(distDirectory, `${filepath.replace(new RegExp(CONFIG_FILE_EXTENSIONS.join('|')), '')}.js`);

  if (configFilePath) {
    const configHelperFilePath = path.normalize(path.join(distDirectory, './config-helper.js'));
    const helperCode = `
      export const defineConfig = (config) => config;
    `;
    await fs.ensureDir(distDirectory);
    await fs.writeFile(configHelperFilePath, helperCode);
    await bundle(configFilePath, _objectSpread(_objectSpread({}, options), {}, {
      getOutputFile,
      esbuildPlugins: [{
        name: 'native-build-config',

        setup(ctx) {
          ctx.onResolve({
            filter: /app-tools\/server/
          }, () => {
            return {
              path: configHelperFilePath
            };
          });
        }

      }]
    }));
  }
};
/**
 *
 * 处理循环引用的 replacer
 */

export const safeReplacer = () => {
  const cache = [];
  const keyCache = [];
  return function (key, value) {
    if (typeof value === 'object' && value !== null) {
      const index = cache.indexOf(value);

      if (index !== -1) {
        return `[Circular ${keyCache[index]}]`;
      }

      cache.push(value);
      keyCache.push(key || 'root');
    }

    return value;
  };
};
export const emitResolvedConfig = async (appDirectory, resolvedConfig) => {
  var _resolvedConfig$outpu;

  const outputPath = path.join(appDirectory, (resolvedConfig === null || resolvedConfig === void 0 ? void 0 : (_resolvedConfig$outpu = resolvedConfig.output) === null || _resolvedConfig$outpu === void 0 ? void 0 : _resolvedConfig$outpu.path) || './dist', OUTPUT_CONFIG_FILE);
  await fs.writeJSON(outputPath, resolvedConfig, {
    spaces: 2,
    replacer: safeReplacer()
  });
};