"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dev = void 0;

var _utils = require("@modern-js/utils");

var _core = require("@modern-js/core");

var _createCompiler = require("../utils/createCompiler");

var _createServer = require("../utils/createServer");

var _routes = require("../utils/routes");

var _printInstructions = require("../utils/printInstructions");

var _getSpecifiedEntries = require("../utils/getSpecifiedEntries");

var _config = require("../utils/config");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const dev = async (api, options) => {
  let userConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const hookRunners = api.useHookRunners();
  userConfig = _objectSpread(_objectSpread({}, userConfig), {}, {
    cliOptions: options
  });

  _core.ResolvedConfigContext.set(userConfig);

  const {
    appDirectory,
    distDirectory,
    port,
    apiOnly,
    entrypoints,
    serverConfigFile
  } = appContext;
  const checkedEntries = await (0, _getSpecifiedEntries.getSpecifiedEntries)(options.entry || false, entrypoints);
  api.setAppContext(_objectSpread(_objectSpread({}, appContext), {}, {
    checkedEntries
  }));
  appContext.checkedEntries = checkedEntries;

  _utils.fs.emptyDirSync(distDirectory);

  await (0, _config.buildServerConfig)({
    appDirectory,
    distDirectory,
    configFile: serverConfigFile,
    options: {
      esbuildOptions: {
        watch: true
      }
    }
  });
  await hookRunners.beforeDev();
  let compiler = null;

  if (!apiOnly) {
    const {
      getWebpackConfig,
      WebpackConfigTarget
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('@modern-js/webpack')));
    const webpackConfigs = [(0, _utils.isSSR)(userConfig) && getWebpackConfig(WebpackConfigTarget.NODE, appContext, userConfig), getWebpackConfig(WebpackConfigTarget.CLIENT, appContext, userConfig)].filter(Boolean);
    compiler = await (0, _createCompiler.createCompiler)({
      api,
      webpackConfigs,
      userConfig,
      appContext
    });
  }

  await (0, _routes.generateRoutes)(appContext);
  const app = await (0, _createServer.createServer)({
    dev: _objectSpread(_objectSpread({}, {
      client: {
        port: port.toString()
      },
      devMiddleware: {
        writeToDisk: file => !file.includes('.hot-update.')
      },
      hot: true,
      liveReload: true,
      port,
      https: userConfig.dev.https
    }), userConfig.tools.devServer),
    compiler,
    pwd: appDirectory,
    config: userConfig,
    serverConfigFile,
    plugins: appContext.plugins.filter(p => p.server).map(p => p.server)
  });
  app.listen(port, async err => {
    if (err) {
      throw err;
    }

    if (apiOnly) {
      return (0, _printInstructions.printInstructions)(hookRunners, appContext, userConfig);
    }

    return _utils.logger.log(_utils.chalk.cyan(`Starting the development server...`));
  });
};

exports.dev = dev;