"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.printInspectResult = exports.inspect = exports.getTagByWebpackTarget = exports.formatWebpackConfig = void 0;

var _path = _interopRequireDefault(require("path"));

var _webpack = require("@modern-js/webpack");

var _utils = require("@modern-js/utils");

var _webpackChain = _interopRequireDefault(require("@modern-js/utils/webpack-chain"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const formatWebpackConfig = (config, verbose) => {
  const stringify = _webpackChain.default.toString;
  return `module.exports = ${stringify(config, {
    verbose
  })};`;
};

exports.formatWebpackConfig = formatWebpackConfig;

const inspect = (api, options) => {
  process.env.NODE_ENV = options.env;
  const resolvedConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const outputFiles = [];
  outputFiles.push(printInspectResult(_webpack.WebpackConfigTarget.CLIENT, appContext, resolvedConfig, options));

  if (resolvedConfig.output.enableModernMode) {
    outputFiles.push(printInspectResult(_webpack.WebpackConfigTarget.MODERN, appContext, resolvedConfig, options));
  }

  if ((0, _utils.isUseSSRBundle)(resolvedConfig)) {
    outputFiles.push(printInspectResult(_webpack.WebpackConfigTarget.NODE, appContext, resolvedConfig, options));
  }

  _utils.logger.success('Inspect succeed, you can open following files to view the full webpack config: \n');

  outputFiles.forEach(file => {
    _utils.logger.log(`  - ${_utils.chalk.yellow(_path.default.relative(appContext.appDirectory, file))}`);
  });

  _utils.logger.log();
};

exports.inspect = inspect;

const getTagByWebpackTarget = webpackTarget => {
  switch (webpackTarget) {
    case _webpack.WebpackConfigTarget.CLIENT:
      return 'client';

    case _webpack.WebpackConfigTarget.MODERN:
      return 'modern';

    case _webpack.WebpackConfigTarget.NODE:
      return 'ssr';

    default:
      throw Error(`Unsupported webpack target: ${webpackTarget}`);
  }
};

exports.getTagByWebpackTarget = getTagByWebpackTarget;

const printInspectResult = (webpackTarget, appContext, resolvedConfig, options) => {
  const webpackConfig = (0, _webpack.getWebpackConfig)(webpackTarget, appContext, resolvedConfig);
  const {
    output,
    verbose,
    console = true
  } = options;
  const outputPath = output ? _path.default.posix.join(appContext.distDirectory, output) : appContext.distDirectory;
  const tag = getTagByWebpackTarget(webpackTarget);
  const outputFile = `webpack.${tag}.inspect.js`;

  const outputFilePath = _path.default.posix.join(outputPath, outputFile);

  const rawWebpackConfig = formatWebpackConfig(webpackConfig, verbose);

  _utils.fs.outputFileSync(outputFilePath, rawWebpackConfig);

  if (console) {
    _utils.logger.log(`
webpack config for ${tag} build:
${rawWebpackConfig}
    `);
  }

  return outputFilePath;
};

exports.printInspectResult = printInspectResult;