"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.safeReplacer = exports.emitResolvedConfig = exports.defineServerConfig = exports.buildServerConfig = void 0;

var path = _interopRequireWildcard(require("path"));

var _nodeBundleRequire = require("@modern-js/node-bundle-require");

var _utils = require("@modern-js/utils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const defineServerConfig = config => config;

exports.defineServerConfig = defineServerConfig;

const buildServerConfig = async ({
  appDirectory,
  distDirectory,
  configFile,
  options
}) => {
  const configFilePath = await (0, _utils.getServerConfig)(appDirectory, configFile);

  const getOutputFile = async filepath => path.resolve(distDirectory, `${filepath.replace(new RegExp(_utils.CONFIG_FILE_EXTENSIONS.join('|')), '')}.js`);

  if (configFilePath) {
    const configHelperFilePath = path.normalize(path.join(distDirectory, './config-helper.js'));
    const helperCode = `
      export const defineConfig = (config) => config;
    `;
    await _utils.fs.ensureDir(distDirectory);
    await _utils.fs.writeFile(configHelperFilePath, helperCode);
    await (0, _nodeBundleRequire.bundle)(configFilePath, _objectSpread(_objectSpread({}, options), {}, {
      getOutputFile,
      esbuildPlugins: [{
        name: 'native-build-config',

        setup(ctx) {
          ctx.onResolve({
            filter: /app-tools\/server/
          }, () => {
            return {
              path: configHelperFilePath
            };
          });
        }

      }]
    }));
  }
};
/**
 *
 * 处理循环引用的 replacer
 */


exports.buildServerConfig = buildServerConfig;

const safeReplacer = () => {
  const cache = [];
  const keyCache = [];
  return function (key, value) {
    if (typeof value === 'object' && value !== null) {
      const index = cache.indexOf(value);

      if (index !== -1) {
        return `[Circular ${keyCache[index]}]`;
      }

      cache.push(value);
      keyCache.push(key || 'root');
    }

    return value;
  };
};

exports.safeReplacer = safeReplacer;

const emitResolvedConfig = async (appDirectory, resolvedConfig) => {
  var _resolvedConfig$outpu;

  const outputPath = path.join(appDirectory, (resolvedConfig === null || resolvedConfig === void 0 ? void 0 : (_resolvedConfig$outpu = resolvedConfig.output) === null || _resolvedConfig$outpu === void 0 ? void 0 : _resolvedConfig$outpu.path) || './dist', _utils.OUTPUT_CONFIG_FILE);
  await _utils.fs.writeJSON(outputPath, resolvedConfig, {
    spaces: 2,
    replacer: safeReplacer()
  });
};

exports.emitResolvedConfig = emitResolvedConfig;