"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCompiler = void 0;

var _webpack = require("@modern-js/webpack");

var _utils = require("@modern-js/utils");

var _printInstructions = require("./printInstructions");

const createCompiler = async ({
  api,
  webpackConfigs,
  // TODO: params
  userConfig,
  appContext
}) => {
  try {
    const hookRunners = api.useHookRunners();
    await hookRunners.beforeCreateCompiler({
      webpackConfigs
    });
    const compiler = (0, _webpack.webpack)(webpackConfigs);
    await hookRunners.afterCreateCompiler({
      compiler
    });
    let isFirstCompile = true;
    compiler.hooks.invalid.tap('invalid', () => {
      (0, _utils.clearConsole)();

      _utils.logger.log('Compiling...');
    });
    compiler.hooks.done.tap('done', async stats => {
      const statsData = stats.toJson({
        preset: 'errors-warnings'
      });
      const {
        errors,
        warnings
      } = (0, _utils.formatWebpackMessages)(statsData);

      if (errors.length) {
        _utils.logger.log(_utils.chalk.red(`Failed to compile.\n`));

        _utils.logger.log(errors.join('\n\n'));

        _utils.logger.log();
      } else if (process.stdout.isTTY || isFirstCompile) {
        await hookRunners.afterDev();

        if (warnings.length) {
          _utils.logger.log(_utils.chalk.yellow(`Compiled with warnings.\n`));

          _utils.logger.log(warnings.join('\n\n'));

          _utils.logger.log();
        }

        await (0, _printInstructions.printInstructions)(hookRunners, appContext, userConfig);
      }

      isFirstCompile = false;
    });
    return compiler;
  } catch (err) {
    _utils.logger.log(_utils.chalk.red(`Failed to compile.`));

    _utils.logger.log();

    _utils.logger.log(err); // FIXME: 这里最好抛出异常，执行 process.exit 的地方尽可能少或者控制在几个统一的地方比较合适
    // eslint-disable-next-line no-process-exit


    process.exit(1);
  }
};

exports.createCompiler = createCompiler;